#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - disk-lib.pl
#
#    Webmin Sysstats Module
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#    $Id: disk-lib.pl,v 1.8 2001/12/07 10:25:55 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  $my_config{'db_uid'} = 1;
  $my_config{'sample_rate'} = 1;

  &write_config($minfo, \%my_config, $defaults);

  &install_script($minfo, 'disk.sh');

  return 1;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if (-r $minfo->{'config_dir'} . "../disk/config.cfg") {
    return 1;
  }
  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local(%old_config);

  if (-r $minfo->{'config_dir'} . "../disk/config.cfg" ) {

    my $cmd = "cp -f " . $minfo->{'dbtables'} . "../disk/*.rrd " . $minfo->{'dbtables'};
    system($cmd);

    if (!&read_file($minfo->{'config_dir'} . "../disk/config.cfg", \%old_config)) {
      &error(&text('ecannot_readconfig', $minfo->{'config_dir'} . "../disk/config.cfg"));
    }
    &write_config($minfo, \%old_config, $defaults);
  }
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  $rrd_detals  = " -l 0  -u 100 -v '% utilization'";
  $rrd_detals .= " COMMENT:'Partition              Max      Min  Average\\n'";

  my $vol_num = 1;
  while ($my_config->{'vol' . $vol_num} ne '') {
    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $vol_num});

    if (!$display) {
      $vol_name .= substr(".................", 0, (0 - length($vol_name)));
      $rrd_detals .= ' DEF:total' . $vol_num . '=' . $minfo->{'dbtables'} . $vol_id . '.rrd:total:AVERAGE';
      $rrd_detals .= ' DEF:used' . $vol_num . '=' . $minfo->{'dbtables'} . $vol_id . '.rrd:used:AVERAGE';
      $rrd_detals .= ' CDEF:med' . $vol_num . '=used' . $vol_num . ',100,*,total' . $vol_num . ',/';
      $rrd_detals .= ' LINE2:med' . $vol_num . '#' . $vol_colour . ':' . $vol_name;
      $rrd_detals .= ' GPRINT:med' . $vol_num . ':MAX:"%6.2lf"';
      $rrd_detals .= ' GPRINT:med' . $vol_num . ':MIN:"%6.2lf"';
      $rrd_detals .= ' GPRINT:med' . $vol_num . ':AVERAGE:"%6.2lf\\n"';
    }
    $vol_num++;
  }

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'}) {
    if ($in{'opt'} eq 'up') {
      my $tmp = $my_config->{'vol' . $in{'vol_num'}};
      $my_config->{'vol' . $in{'vol_num'}} = $my_config->{'vol' . ($in{'vol_num'} - 1)};
      $my_config->{'vol' . ($in{'vol_num'} - 1)} = $tmp;
    } elsif ($in{'opt'} eq 'down') {
      my $tmp = $my_config->{'vol' . $in{'vol_num'}};
      $my_config->{'vol' . $in{'vol_num'}} = $my_config->{'vol' . ($in{'vol_num'} + 1)};
      $my_config->{'vol' . ($in{'vol_num'} + 1)} = $tmp;
    } elsif ($in{'opt'} eq 'runstop') {
      local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
	= split(',', $my_config->{'vol' . $in{'vol_num'}});

      $runstop = $runstop ? 0 : 1;
      $my_config->{'vol' . $in{'vol_num'}}
	= join(',', $vol_id, $vol_name, $vol_colour, $runstop, $display);
    } elsif ($in{'opt'} eq 'display') {
      local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
	= split(',', $my_config->{'vol' . $in{'vol_num'}});

      $display = $display ? 0 : 1;
      $my_config->{'vol' . $in{'vol_num'}}
	= join(',', $vol_id, $vol_name, $vol_colour, $runstop, $display);
    } elsif ($in{'opt'} eq 'rm') {
      my $vol_num = $in{'vol_num'};

      local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
	= split(',', $my_config->{'vol' . $vol_num});

      while ($my_config->{'vol' . $vol_num} ne '') {
	$my_config->{'vol' . $vol_num++} = $my_config->{'vol' . ($vol_num)};
      }

      $my_config->{'vol' . ($vol_num - 1)} = '';

      &write_config($minfo, $my_config, $defaults);

      unlink ($minfo->{'dbtables'} . $vol_id . '.rrd');

    }

    &write_config($minfo, $my_config, $defaults);
  }

  print "<table border width=100%>\n";
  print  "<tr $cb>";
  print   "<td>";
  print    "<table width=100%>\n";
  print     "<tr><td><b> $text{'module_subtitle'}</b></td></tr>\n";

  print     "<tr><td><table border width=100%>\n";
  print     "<td $tb><b>$text{'module_subtitle_vol_name'}</b></td>";
  print     "<td $tb><b>$text{'module_subtitle_disp_colour'}</b></td>";
  print     "<td $tb width=" . (22 * 6) . "><b>$text{'module_subtitle_controls'}</b></td>";

  my $vol_num = 1;
  while ($my_config->{'vol' . $vol_num} ne '') {
    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $vol_num});

    print    "<tr $cb>";
    print     "<td><b><a href='edit_module.cgi?idx=$in{'idx'}&opt=edit&vol_num=$vol_num'>$vol_name</a></b></td>";
    print     "<td><font color=$vol_colour><b>";
    print     $text{'colour_' . $vol_colour} ? $text{'colour_' . $vol_colour} : "<pre>#$vol_colour </pre>";
    print     "</b></font></pre></td>";
    print     "<td align=right>";
    if ($access{'configure'}) {
      if ($vol_num > 1) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=up&vol_num=$vol_num'>";
	print "<IMG src='images/up.png' border=0>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border=0>";
	print "<IMG src='images/space.png' border=0>";
      }
      print "<IMG src='images/space.png' border=0>";
      if ($my_config->{'vol' . ($vol_num + 1)} ne '') {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=down&vol_num=$vol_num'>";
	print "<IMG src='images/down.png' border=0>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border=0>";
	print "<IMG src='images/space.png' border=0>";
      }
      print "<IMG src='images/space.png' border=0>";
      if ($runstop) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&vol_num=$vol_num'>";
	print "<IMG src='images/record.png' border=0>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&vol_num=$vol_num'>";
	print "<IMG src='images/pause.png' border=0>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border=0>";
      if ($display) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display&vol_num=$vol_num'>";
	print "<IMG src='images/nodata.png' border=0>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display&vol_num=$vol_num'>";
	print "<IMG src='images/showdata.png' border=0>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border=0>";
      print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=rm&vol_num=$vol_num'>";
      print "<IMG src='images/trash.png' border=0>";
      print "</a>";
    }
    print    "</td>";
    $vol_num++;
  }
  print "</table></td></tr>";

  if ($in{'opt'} eq 'edit') {
    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $in{'vol_num'}});

    print     "<tr>\n";
    print      "<td>";
    print       "<table border width=100%>\n";
    print        "<tr $tb><td><b>" . $text{'module_vol'} . "</b></td></tr>\n";
    print        "<tr $cb>";
    print         "<td>";
    print          "<form action=save_config.cgi>";
    print          "<table width=100%>\n";
    print           "<tr>\n";
    print            "<td width=30%><b>" . $text{'module_word_vol'} . "</b></td>\n";
    print             "<td nowrap><b>$vol_name</b></td>";
    print            "</tr><tr>";
    print             "<td><b>" . $text{'word_colour'} . "</b></td>\n";
    print             "<td>";
    &option_colourselect('vol_colour', $vol_colour);
    print             "</td>";
    print            "</tr><tr>\n";
    print            "<td><b>" . $text{'word_display_data'} . "</b></td>";
    printf "<td><input type=radio name=display_def value=0 %s> $text{'word_yes'}  ",
      !$display ? 'checked' : '' ;
    printf "<input type=radio name=display_def value=1 %s> $text{'word_no'}</td></tr>\n",
      $display ? 'checked' : '';
    print            "<tr><td><b>" . $text{'module_pause_datacapture'} . "</b></td>";
    printf "<td><input type=radio name=runstop_def value=0 %s> $text{'word_yes'}  ",
      !$runstop ? 'checked' : '' ;
    printf "<input type=radio name=runstop_def value=1 %s> $text{'word_no'}</td></tr>\n",
      $runstop ? 'checked' : '';
    print           "<tr>";
    print            "<td align=left>";
    print              "<input type=hidden name=idx value='" . $in{'idx'} . "'>";
    print              "<input type=hidden name=opt value='update_vol'>";
    print              "<input type=hidden name=vol value='" . $in{'vol_num'} . "'>";
    print              "<input type=submit value='Update:'>";
    print             "</form>";
    print            "</td>";
    print           "</tr>";
    print          "</table>";
    print         "</td>";
    print        "</tr>";
    print       "</table>";
    print      "</td>";
    print     "</tr>\n";
  } else {

    @vol_list = split ( /\n/, `df | grep -v Filesystem | awk '{print \$6}'`);

    $i = 0;
    print     "<tr>\n";
    print      "<td>";
    print       "<table border width=100%>\n";
    print        "<tr $tb><td><b>" . $text{'module_new_vol'} . "</b></td></tr>\n";
    print        "<tr $cb>";
    print         "<td>";
    print          "<form action=save_config.cgi>";
    print          "<table width=100%>\n";
    print           "<tr>\n";
    print            "<td width=30%><b>" . $text{'module_word_vol'} . "</b></td>\n";
    print            "<td nowrap><b>";
    print              "<select name='new_vol_def'>\n";
    printf              "<option value='" . $i++ . "'>%s\n", $text{'word_select'};
    foreach $vlist (@vol_list) {
      printf            "<option value='" . $i++ . "'>%s\n", $vlist;
    }
    print              "</select>";
    print             "</b></td>";
    print            "</tr><tr>";
    print             "<td><b>" . $text{'word_colour'} . "</b></td>\n";
    print             "<td>";
    &option_colourselect('new_vol_colour', '');
    print             "</td>";
    print            "</tr><tr>";
    print            "<td><b>" . $text{'word_display_data'} . "</b></td>";
    print "<td><input type=radio name=new_display_def value=1 checked> $text{'word_yes'}  ";
    print "<input type=radio name=new_display_def value=0> $text{'word_no'}</td>";
    print           "</tr>\n";
    print           "<tr>";
    print            "<td align=left>";
    print              "<input type=hidden name=idx value='" . $in{'idx'} . "'>";
    print              "<input type=hidden name=opt value='new_vol'>";
    print              "<input type=hidden name=vol value='" . $vol_num . "'>";
    print              "<input type=submit value='Add New:'>";
    print             "</form>";
    print            "</td>";
    print           "</tr>";
    print          "</table>";
    print         "</td>";
    print        "</tr>";
    print       "</table>";
    print      "</td>";
    print     "</tr>\n";
  }
  print    "</table>";
  print   "</td>";
  print  "</tr>";
  print "</table>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'} eq 'new_vol') {
    if (($in{'new_vol_def'} eq '0') ||
	(($in{'new_vol_colour1_def'} eq 'nocolour') &&
	 ($in{'new_vol_colour2_def'} eq ''))) {
      &error($text{'save_einvaled_selection'});
    }

    if (!-r $minfo->{'dbtables'} . $my_config->{'db_uid'} . '.rrd') {
      local ($cmd);

      $cmd = $config{'rrdtool'};
      $cmd .= " create " . $minfo->{'dbtables'} . $my_config->{'db_uid'} . '.rrd';
      $cmd .= " --start 0 --step 60";
      $cmd .= " DS:total:GAUGE:100:U:U";
      $cmd .= " DS:used:GAUGE:100:U:U";
      $cmd .= " RRA:AVERAGE:0.5:1:70";
      $cmd .= " RRA:AVERAGE:0.5:2:750";
      $cmd .= " RRA:AVERAGE:0.5:15:650";
      $cmd .= " RRA:AVERAGE:0.5:50:650";

      `$cmd`;
    }

    @vol_list = split ( /\n/, `df | grep -v Filesystem | awk '{print \$6}'`);
    $my_config->{'vol' . $in{'vol'}} =
      join (',', $my_config->{'db_uid'}++,
	    @vol_list[$in{'new_vol_def'} - 1],
	    $in{'new_vol_colour2_def'} ? $in{'new_vol_colour2_def'} : $in{'new_vol_colour1_def'},
	    0, $in{'new_display_def'} ? '0' : '1'
	   );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'del_vol') {
    my $vol_num = $in{'vol'};

    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $in{'vol'}});

    while ($my_config->{'vol' . $vol_num} ne '') {
      $my_config->{'vol' . $vol_num} = $my_config->{'vol' . ($vol_num + 1)};
      $vol_num++;
    }

    $my_config->{'vol' . ($vol_num - 1)} = '';

    &write_config($minfo, $my_config, $defaults);

    unlink ($minfo->{'dbtables'} . $vol_id . '.rrd');

    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'update_vol') {
    if (($in{'vol_colour1_def'} eq 'nocolour') &&
	 ($in{'vol_colour2_def'} eq '')) {
      &error($text{'save_einvaled_selection'});
    }

    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $in{'vol'}});

    $my_config->{'vol' . $in{'vol'}} =
      join (',', $vol_id,
	    $vol_name,
	    $in{'vol_colour2_def'} ? $in{'vol_colour2_def'} : $in{'vol_colour1_def'},
	    $in{'runstop_def'},
	    $in{'display_def'}
	   );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  }
  &error($in);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};

  $new_config{'db_uid'} = $my_config->{'db_uid'};

  my $i = 1;
  while ($my_config->{'vol' . $i} ne '') {
    $new_config{'vol' . $i} = $my_config->{'vol' . $i};
    $i++;
  }

 $i = 1;
  my $n = 1;
  while ($my_config->{'vol' . $i} ne '') {
    local ($vol_id, $vol_name, $vol_colour, $runstop, $display)
      = split(',', $my_config->{'vol' . $i++});

    if (!$runstop && !$new_config{'runstop'}) {
      $sh_config{'vol' . $n} = '(';
      $sh_config{'vol' . $n} .= $vol_name . ' ';
      $sh_config{'vol' . $n} .= $vol_id;
      $sh_config{'vol' . $n++} .= ')';
    }
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . '/disk.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . '/disk.cfg', \%sh_config);
}


return 1;
